package com.lantone.qc.kernel.catalogue.yiwu.threelevelward;

import com.lantone.qc.dbanaly.util.KernelConstants;
import com.lantone.qc.dbanaly.util.SpecialStorageUtil;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.model.doc.operation.OperationDoc;
import com.lantone.qc.pub.model.doc.ward.DirectorDoctorWardDoc;
import com.lantone.qc.pub.util.DateUtil;
import com.lantone.qc.pub.util.SpringContextUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : THR0125
 * @Description : 副主任医师/主任医师查房首次查房记录未在患者入院72h内完成
 * @Author : 胡敬
 * @Date: 2020-03-19 13:51
 */
@Component
public class THR0125 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        if (inputInfo.getThreeLevelWardDocs().size() == 0) {
            status.set("0");
            return;
        }
        String admisTime = null, dischargeTime = null;
        if (inputInfo.getLeaveHospitalDoc() != null) {
            if (inputInfo.getMedicalRecordInfoDoc() != null) {
                Map<String, String> structureMap = inputInfo.getMedicalRecordInfoDoc().getStructureMap();
                admisTime = structureMap.get("behospitalDate");
                dischargeTime = structureMap.get("leaveHospitalDate");
            }
            if (CatalogueUtil.isEmpty(admisTime)) {
                status.set("0");
                return;
            }
            //如果住院天数小于3天则不判断该条规则
            if (DateUtil.parseDate(dischargeTime) != null && StringUtil.isNotEmpty(dischargeTime) &&
                    !CatalogueUtil.compareTime(StringUtil.parseDateTime(admisTime), StringUtil.parseDateTime(dischargeTime), (long) (72 * 60))) {
                status.set("0");
                return;
            }
        }
        if (inputInfo.getBeHospitalizedDoc() != null) {
            Map<String, String> beHospitalStructureMap = inputInfo.getBeHospitalizedDoc().getStructureMap();
            admisTime = beHospitalStructureMap.get("入院日期");
            if (CatalogueUtil.isEmpty(admisTime)) {
                status.set("0");
                return;
            }
            if (!CatalogueUtil.compareTime(
                    StringUtil.parseDateTime(admisTime),
                    StringUtil.parseDateTime(DateUtil.nowString()),
                    72 * 60L)) {//如果入院未超过72小时，规则不判断
                status.set("0");
                return;
            }
            //开始时间(入院时间)
            Date beginDate = StringUtil.parseDateTime(admisTime);
            if (beginDate == null) {
                status.set("0");
                return;
            }
            ThreeLevelWardDoc threeLevelWardDoc = inputInfo.getThreeLevelWardDocs().get(0);
            List<DirectorDoctorWardDoc> directorDoctorWardDocs = threeLevelWardDoc.getDirectorDoctorWardDocs();
            if (directorDoctorWardDocs.size() > 0) {
                DirectorDoctorWardDoc firstDirector = directorDoctorWardDocs.get(0);
                String wardDateStr = firstDirector.getStructureMap().get("查房日期");
                if (!CatalogueUtil.compareTime(StringUtil.parseDateTime(admisTime), StringUtil.parseDateTime(wardDateStr), 72 * 60L)) {
                    status.set("0");
                    return;
                }
            } else if (!CatalogueUtil.compareTime(StringUtil.parseDateTime(admisTime), new Date(), 72 * 60L)) {
                status.set("0");//没有主任医师查房，若入院时间和系统当前时间对比，相差72小时则报错
                return;
            }

            /* 如果存在手术记录，判断主刀医生是否为主治医生 */
            String operatorName = "";
            List<OperationDoc> operationDocs = inputInfo.getOperationDocs();
            if (operationDocs != null) {
                for (OperationDoc operationDoc : operationDocs) {
                    if (operationDoc.getOperationRecordDoc() != null) {
                        Map<String, String> operationDocStructureMap = operationDoc.getOperationRecordDoc().getStructureMap();
                        if (StringUtil.isBlank(operatorName)) {
                            operatorName = operationDocStructureMap.get("主刀医师");
                            if (StringUtil.isBlank(operatorName) && StringUtil.isNotBlank(operationDocStructureMap.get("手术者及助手名称"))) {
                                operatorName = operationDocStructureMap.get("手术者及助手名称").split("、")[0];
                                if (operatorName.contains("主刀：") && operatorName.split("：").length > 1) {
                                    operatorName = operatorName.split("：")[1];
                                }
                            }
                        }
                    }
                }
            }

            List<ThreeLevelWardDoc> allDoctorWradDocs = threeLevelWardDoc.getAllDoctorWradDocs();
            for (ThreeLevelWardDoc threeLevelWard : allDoctorWradDocs) {
                Map<String, String> structureMap = threeLevelWard.getStructureMap();
                String wardDateStr = structureMap.get("查房日期");
                if (!CatalogueUtil.compareTime(StringUtil.parseDateTime(admisTime), StringUtil.parseDateTime(wardDateStr), 72 * 60L)) {
                    String makeTitle = structureMap.get("查房标题");
                    String pathography = structureMap.get("病情记录");
                    //判断标题
                    visitingPhysicianHouse(operatorName, makeTitle);
                    //判断病情记录
                    visitingPhysicianHouse(operatorName, pathography);
                }
            }
        }
    }

    /**
     * 判断查房记录是否含有主任医师查房
     *
     * @param operatorName     主刀医师
     * @param titlePathography 查房标题或病情记录
     */
    private void visitingPhysicianHouse(String operatorName, String titlePathography) {
        if (StringUtil.isNotBlank(titlePathography)) {
            if (titlePathography.contains("主任")) {
                status.set("0");
            } else if (StringUtil.isNotBlank(operatorName) && (titlePathography.contains(operatorName) || titlePathography.contains("主刀"))) {
                String operationProfessor = getCourseProfessor(operatorName);
                if (operationProfessor.contains("主任")) {
                    status.set("0");
                }
            }
        }
    }

    private String getCourseProfessor(String operatorName) {
        String professor = "";
        if (StringUtil.isBlank(operatorName)) {
            return professor;
        }
        SpecialStorageUtil specialStorageUtil = SpringContextUtil.getBean("specialStorageUtil");
        Map<String, Object> surgeon = specialStorageUtil.getJsonStringValue(KernelConstants.HOSPITAL_DOCTOR_MAP);
        if (surgeon != null) {
            Map<String, String> doctor = (Map) surgeon.get(operatorName);
            if (doctor != null) {
                professor = doctor.get("professor");
            }

        }
        return professor;
    }
}